@@ -299,80 +299,6 @@ return(i);
 }*/
 
 
-/// Emit Postscript command for font selection.
-/// @param[in,out]	Text	String builter that holds whole PS image.
-/// @param[in]		FontSize  Size of font in [points]
-/// @param[in]		FontName  Optional naome of font.
-void Font2PS(string & Text, float FontSize, const char *FontName)
-{
-  Text.cat_printf("/%s findfont %2.2f scalefont setfont", FontName, FontSize);
-}
-
-
-/** Render polygonal curve. */
-static void Curve2PS(float *Points, int Count, string & PSData, PS_State *PSS, bool Filled=true, bool Outline=true)
-{
-int i;
-float MinPolyX=65537,MaxPolyX=-1,MinPolyY=65537,MaxPolyY=-1;
-float xi,yi,xa,ya,xt,yt,xt_old,yt_old;
-
-  if(Points==NULL || Count<=0) return;
-
-  PS_Attr(PSData,PSS);
-  for(i=0;i<Count;i++)
-	{
-        xi = WPGu2PSu(*Points++);
-	yi = WPGu2PSu(*Points++);
-	xa = WPGu2PSu(*Points++);
-	ya = WPGu2PSu(*Points++);
-        xt = WPGu2PSu(*Points++);
-	yt = WPGu2PSu(*Points++);
-
-	if(xi<MinPolyX) MinPolyX=xi;  if(xi>MaxPolyX) MaxPolyX=xi;
-	if(yi<MinPolyY) MinPolyY=yi;  if(yi>MaxPolyY) MaxPolyY=yi;
-
-	if(xa<MinPolyX) MinPolyX=xa;  if(xa>MaxPolyX) MaxPolyX=xa;
-	if(ya<MinPolyY) MinPolyY=ya;  if(ya>MaxPolyY) MaxPolyY=ya;
-
-	if(xt<MinPolyX) MinPolyX=xt;  if(xt>MaxPolyX) MaxPolyX=xt;
-	if(yt<MinPolyY) MinPolyY=yt;  if(yt>MaxPolyY) MaxPolyY=yt;
-
-	if(i==0)
-	  {
-          PSData.cat_printf("\nnewpath\n%2.2f %2.2f moveto",xa,ya);
-          }
-        else
-	  {
-	  PSData.cat_printf("\n%2.2f %2.2f %2.2f %2.2f %2.2f %2.2f curveto",
-		       xt_old,yt_old,xi,yi,xa,ya);
-          }
-        xt_old=xt;
-        yt_old=yt;
-	}
-
-  if(Filled)
-    {
-    PSData += "\nclosepath";
-    i = PSS->LineStyle;
-    if(!Outline) PSS->LineStyle=0;  //temporarily disable line
-    FillObjectPS(MinPolyX,MaxPolyX,MinPolyY,MaxPolyY,PSData,PSS);
-    PSS->LineStyle = i;
-    }
-  else
-    PSData += "\nstroke";
-}
-
-
-/** Store color in PS notation. */
-void Color2PS(string & Text, const RGB_Record & TextColor)
-{
-  Text.cat_printf("%0.2g %0.2g %0.2g setrgbcolor",
-	         (float)TextColor.Red/256.0,
-		 (float)TextColor.Green/256.0,
-	         (float)TextColor.Blue/256.0);
-}
-
-
 /** Convert a block of WP5.x text into PostScript. */
 static void Textl2_2PS(TconvertedPass1 *cq, long ldblk, string & PSData, PS_State *PSS, WPG_records *WPG)
 {
@@ -635,7 +561,7 @@ long ActualPos;
 
 
 /** Load array of points word packed. */
-float *LoadPoints(TconvertedPass1 *cq, int n, FloatBBox &bbx, WPG2_Transform *TRx, bool Precision)
+float *LoadPoints(TconvertedPass1 *cq, int n, FloatBBox &bbx, AbstractTransformXY *TRx, bool Precision)
 {
 #ifdef DEBUG
   fprintf(cq->log,"\n#LoadPoints(%d) ",n);fflush(cq->log);
@@ -773,6 +699,25 @@ static void BuildTM(const WPG2_Transform &TRx, float_matrix &TM, bool rotations=
 }
 
 
+/** This functor class is used for resizing image data. */
+class vecResizeCTM: public AbstractTransformXY, public WPG2_Transform
+{
+public:
+  vecResizeCTM(void) {}
+
+  virtual void ApplyTransform(float &x, float &y) const;
+};
+
+
+void vecResizeCTM::ApplyTransform(float &x, float &y) const
+{
+const float NewX = 
+     ScaleX*x + SkewX*y  + TranslateX;
+ y = SkewY*x  + ScaleY*y + TranslateY;
+ x = NewX;
+}
+
+
 class TImgConvStatus
 {
 public:
@@ -808,7 +753,7 @@ void TImgConvStatus::ClearBrushes(void)
 
 /** Processing one token inside separate procedure allows to recurse tokens. */
 static void ProcessWPG2Token(TconvertedPass1 *cq, WPG2Record & Rec2,
-      WPG2Start & StartWPG, Image & Img, float_matrix & CTM, APalette **Palette,
+      WPG2Start & StartWPG, Image & Img, /*float_matrix & CTM,*/ APalette **Palette,
       string & PSData, PS_State & PSS, EmbeddedImg **EmImg,
       DWORD & NewObject, DWORD & ResourceEnd, Image **CurrImg,
       TImgConvStatus & ImgSts, FloatBBox & bbx, VectorList &VectList)
@@ -820,7 +765,7 @@ WPG_records WPG;
 WORD i;
 Raster2DAbstract *Raster = NULL;
 long ldblk;
-WPG2_Transform TRx;
+vecResizeCTM TRx;
 
   switch(Rec2.Type)
     {
@@ -954,42 +899,26 @@ WPG2_Transform TRx;
                if(ImgSts.dx>=0)
                    {(*CurrImg)->x = ImgSts.x; (*CurrImg)->dx = ImgSts.dx;}
                else
-                   {(*CurrImg)->x = ImgSts.x+ImgSts.dx; (*CurrImg)->dx = -ImgSts.dx;}
+               {
+                 (*CurrImg)->x = ImgSts.x; (*CurrImg)->dx = -ImgSts.dx;
+                 if(Raster!=NULL) Flip1D(Raster);
+               }
                if(ImgSts.dy>=0)
                    {(*CurrImg)->y = ImgSts.y; (*CurrImg)->dy = ImgSts.dy;}
                else
-                   {(*CurrImg)->y = ImgSts.y+ImgSts.dy; (*CurrImg)->dy = -ImgSts.dy;}
+               {
+                 (*CurrImg)->y = ImgSts.y; (*CurrImg)->dy = -ImgSts.dy;
+                 if(Raster!=NULL) Flip2D(Raster);
+               }
 
                (*CurrImg)->RotAngle = ImgSts.RotAngle;
 	       ImgSts.x = ImgSts.y = ImgSts.dx = ImgSts.dy = 0;
 	       if(Raster!=NULL)
 	       {
-		    if(CTM.member(0,0)<0)
-		      {
-		      Flip1D(Raster); //?? RotAngle=360-RotAngle;
-			/* Try to change CTM according to Flip - I am not sure, must be checked. */
-		      float_matrix Tx(3,3);
-		      Tx(0,0)=-1;      Tx(1,0)=0;   Tx(2,0)=0;
-		      Tx(0,1)= 0;      Tx(1,1)=1;   Tx(2,1)=0;
-		      Tx(0,2)=(WPG._2Rect.X_ur+WPG._2Rect.X_ll);
-				       Tx(1,2)=0;   Tx(2,2)=1;
-		      CTM *= Tx;
-		      }
-		    if(CTM.member(1,1)<0)
-		      {
-		      Flip2D(Raster); //?? RotAngle=360-RotAngle;
-			  /* Try to change CTM according to Flip - I am not sure, must be checked. */
-		      float_matrix Tx(3,3);
-		      Tx(0,0)= 1;   Tx(1,0)= 0;   Tx(2,0)=0;
-		      Tx(0,1)= 0;   Tx(1,1)=-1;   Tx(2,1)=0;
-		      Tx(0,2)= 0;   Tx(1,2)=(WPG._2Rect.Y_ur+WPG._2Rect.Y_ll);
-						  Tx(2,2)=1;
-		      CTM *= Tx;
-		      }
-		    (*CurrImg)->Raster=Raster;(*CurrImg)->Raster->UsageCount++;Raster=NULL;
-		    }
-		  AssignPalette(*CurrImg,*Palette);
-		  break;
+		  (*CurrImg)->Raster=Raster;(*CurrImg)->Raster->UsageCount++;Raster=NULL;
+	       }
+	       AssignPalette(*CurrImg,*Palette);
+	       break;
     case 0x0F:{//CrackObject(cq,NewObject);
               UpdateBBox(bbx, 0,
 		         WPGu2PSu(ImgSts.x),WPGu2PSu(ImgSts.y),WPGu2PSu(ImgSts.x+ImgSts.dx),WPGu2PSu(ImgSts.y+ImgSts.dy));
@@ -1019,9 +948,8 @@ WPG2_Transform TRx;
     case 0x15:{strcpy(cq->ObjType,"Polyline");
 		  float *Points;
 
-		  LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+		  TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 
-			//!!RotAngle Ignored!!!
 		  Rd_word(cq->wpd,&WPG.Curve.Count);
 		  FixPolySize(cq, &WPG.Curve.Count, (StartWPG.PosSizePrecision==0)?Rec2.RecordLength:(Rec2.RecordLength/2)); //insufficient fix -sizeof(CTM)!!!
 		  Points = LoadPoints(cq, WPG.Curve.Count, bbx, &TRx, StartWPG.PosSizePrecision>0);
@@ -1034,17 +962,20 @@ WPG2_Transform TRx;
                       pVecPoly->AttribFromPSS(PSS);  
                       pVecPoly->Close = true;
                       pVecPoly->Outline = TRx.Flags&WPG2_Transform::WPG2_FRM;
+                      if((TRx.Flags & WPG2_Transform::WPG2_FRM) == 0) pVecPoly->LineStyle=0;	// Disable object fill.
+                      else
+                          {if(pVecPoly->LineStyle==0) pVecPoly->LineStyle=1;}                      
                       pVecPoly->Transform(vecResizeXY(WPGu2PSu(1)));
-                      VectList.AddObject(pVecPoly); pVecPoly=NULL;
+                      VectList.AddObject(pVecPoly);
                       }
 		  else
                       {
                       VectorLine *pVecLine = new VectorLine(Points, WPG.Curve.Count);
                       Points = NULL;
                       pVecLine->AttribFromPSS(PSS);  
-                      pVecLine->Close = TRx.Flags&WPG2_Transform::WPG2_CLS;
+                      pVecLine->Close = TRx.Flags&WPG2_Transform::WPG2_CLS;                      
                       pVecLine->Transform(vecResizeXY(WPGu2PSu(1)));
-                      VectList.AddObject(pVecLine); pVecLine=NULL;
+                      VectList.AddObject(pVecLine);
                       }
                    		  
 		  break;
@@ -1053,21 +984,23 @@ WPG2_Transform TRx;
     case 0x17:{strcpy(cq->ObjType,"Polycurve");
 		  float *Points;
 
-		  LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+		  TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 			//!!RotAngle Ignored!!!
 		  Rd_word(cq->wpd,&WPG.Curve.Count);
-		  WPG.Curve.Count*=3;
+		  WPG.Curve.Count *= 3;
 		  FixPolySize(cq, &WPG.Curve.Count, (StartWPG.PosSizePrecision==0)?Rec2.RecordLength:(Rec2.RecordLength/2)); //insufficient fix -sizeof(CTM)!!!
-		  Points = LoadPoints(cq,WPG.Curve.Count,bbx,NULL,StartWPG.PosSizePrecision>0);
-		  WPG.Curve.Count/=3;
+		  Points = LoadPoints(cq,WPG.Curve.Count,bbx,NULL,StartWPG.PosSizePrecision>0);		 
 		  if(Points==NULL) break;
-		  Curve2PS(Points,WPG.Curve.Count,PSData,&PSS,
-			   TRx.Flags&0x2000, TRx.Flags&0x8000); //check Fill + Frame flags
-		  free(Points);
+                  VectorCurve *pVecCurve = new VectorCurve(Points, WPG.Curve.Count);
+                  pVecCurve->AttribFromPSS(PSS);
+                  pVecCurve->Transform(vecResizeXY(WPGu2PSu(1)));		  
+                  if(TRx.Flags&WPG2_Transform::WPG2_FILL) pVecCurve->Filled=true;	// check Fill
+                  if(TRx.Flags&WPG2_Transform::WPG2_FRM) pVecCurve->LineStyle=0;	// Frame flags
+		  VectList.AddObject(pVecCurve);
 		  break;
 		  }
     case 0x18:{strcpy(cq->ObjType,"Rectangle");
-		  LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+		  TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 		  ImgSts.RotAngle = TRx.RotAngle;
 
 		  LoadWPG2Rectangle(cq->wpd,WPG._2Rect);
@@ -1082,34 +1015,46 @@ WPG2_Transform TRx;
 		  }
     case 0x19:{strcpy(cq->ObjType,"Arc");
                   //CrackObject(cq,NewObject);
-		  LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+		  TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 		  LoadWPG2Arc(cq->wpd,WPG._2Arc);
 
-                  TRx.ScaleX = TRx.ScaleY = 1;
-		  TRx.TamperX = TRx.TamperY = TRx.SkewX = TRx.SkewY = TRx.TranslateX = TRx.TranslateY = 0;
+                  float Sx = WPG._2Arc.Sx;
+                  float Sy = WPG._2Arc.Sy;
+                  TRx.ApplyTransform(Sx,Sy);
+                  float LeftX = WPG._2Arc.Sx - WPG._2Arc.rx;
+                  float LeftY = WPG._2Arc.Sy;
+                  TRx.ApplyTransform(LeftX,LeftY);
+                  float TopX = WPG._2Arc.Sx;
+                  float TopY = WPG._2Arc.Sy + WPG._2Arc.ry;
+                  TRx.ApplyTransform(TopX,TopY);
 
-                  float_matrix TM(3,3);
-                  BuildTM(TRx,TM,true);
-                  float_matrix corners(3,4,1.0);
-                  corners.member(0,0)=WPG._2Arc.Sx-WPG._2Arc.rx;    corners.member(1,0)=WPG._2Arc.Sy-WPG._2Arc.ry;
-                  corners.member(0,1)=WPG._2Arc.Sx+WPG._2Arc.rx;    corners.member(1,1)=WPG._2Arc.Sy-WPG._2Arc.ry;
-                  corners.member(0,2)=WPG._2Arc.Sx-WPG._2Arc.rx;    corners.member(1,2)=WPG._2Arc.Sy+WPG._2Arc.ry;
-                  corners.member(0,3)=WPG._2Arc.Sx+WPG._2Arc.rx;    corners.member(1,3)=WPG._2Arc.Sy+WPG._2Arc.ry;
-                  corners = corners*TM;
-                  for(i=0; i<corners.cols(); i++)
-                  {
-                    float f = corners(0,i);
-                    if(bbx.MinX>f) bbx.MinX=f;
-                    if(bbx.MaxX<f) bbx.MaxX=f;
-                    f = corners(1,i);
-                    if(bbx.MinY>f) bbx.MinY=f;
-                    if(bbx.MaxY<f) bbx.MaxY=f;
-                  }
+		  float rx = sqrt((Sx-LeftX)*(Sx-LeftX) + (Sy-LeftY)*(Sy-LeftY));
+                  float ry = sqrt((Sx-TopX)*(Sx-TopX) + (Sy-TopY)*(Sy-TopY));
+                  VectorEllipse *pVecEllipse = new VectorEllipse(Sy-ry, Sy+ry, Sx-rx, Sx+rx);
+                  if(fabs(TRx.RotAngle) > 1e-3)
+                    {
+                    pVecEllipse->Tx = new vecTransform;
+                    pVecEllipse->Tx->RotAngle = TRx.RotAngle;
+                    pVecEllipse->Tx->CenterX = Sx;
+                    pVecEllipse->Tx->CenterY = Sy;
+                    }
+                  if((WPG._2Arc.Xi!=WPG._2Arc.Xt || WPG._2Arc.Yi!=WPG._2Arc.Yt) &&
+                     WPG._2Arc.rx!=0 && WPG._2Arc.ry!=0)
+                    {
+                    pVecEllipse->bAngle = 180*atan2(WPG._2Arc.Yi/(float)WPG._2Arc.ry, WPG._2Arc.Xi/(float)WPG._2Arc.rx)/M_PI;
+                    pVecEllipse->eAngle = 180*atan2(WPG._2Arc.Yt/(float)WPG._2Arc.ry, WPG._2Arc.Xt/(float)WPG._2Arc.rx)/M_PI;
+                    if(pVecEllipse->bAngle<0) pVecEllipse->bAngle+=360;
+                    if(pVecEllipse->eAngle<0) pVecEllipse->eAngle+=360;
+                    }
 
-                  VectorEllipse *pVecEllipse = new VectorEllipse(WPGu2PSu(WPG._2Arc.Sy-WPG._2Arc.ry), WPGu2PSu(WPG._2Arc.Sy+WPG._2Arc.ry),
-								 WPGu2PSu(WPG._2Arc.Sx-WPG._2Arc.rx), WPGu2PSu(WPG._2Arc.Sx+WPG._2Arc.rx));
-                  pVecEllipse->Tx = BuildTransform(TRx, WPG._2Arc.Sx, WPG._2Arc.Sy);
-                  pVecEllipse->AttribFromPSS(PSS);		  
+                  UpdateBBox(bbx, TRx.RotAngle, Sx-rx, Sy-ry, 2*rx, 2*ry);                  
+                  pVecEllipse->Transform(vecResizeXY(WPGu2PSu(1)));
+
+                  pVecEllipse->AttribFromPSS(PSS);
+                  if((TRx.Flags & WPG2_Transform::WPG2_FILL) == 0) pVecEllipse->BrushStyle=0;	// Disable object fill.
+                  if((TRx.Flags & WPG2_Transform::WPG2_FRM) == 0) pVecEllipse->LineStyle=0;	// Disable object fill.
+                  else
+                    {if(pVecEllipse->LineStyle==0) pVecEllipse->LineStyle=1;}
                   VectList.AddObject(pVecEllipse);
 		  break;
 		  }
@@ -1117,35 +1062,54 @@ WPG2_Transform TRx;
     case 0x1A:strcpy(cq->ObjType,"!Compound Polygon");break;
     case 0x1B:{strcpy(cq->ObjType,"Bitmap position");
 	      //CrackObject(cq,NewObject);//!!!!
-
-	      LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
-	      ImgSts.RotAngle = TRx.RotAngle;
-              BuildTM(TRx, CTM, false);              
+	      TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);	      
               //printf("\n Rot angle = %f    ", RotAngle);
 
+              float UpRightX, UpRightY;
+              float Sx, Sy;
+              float RightX, RightY;
+              float TopX, TopY;                  
 	      if(StartWPG.PosSizePrecision==0)
 		 {
 		 LoadWPG2BitmapRectangle(cq->wpd,WPG._2BitmapRectangle);
-		 ImgSts.x = WPG._2BitmapRectangle.LowLeftX;
-		 ImgSts.y = WPG._2BitmapRectangle.LowLeftY;
-		 ImgSts.dx = (WPG._2BitmapRectangle.UpRightX-WPG._2BitmapRectangle.LowLeftX);
-		 ImgSts.dy = (WPG._2BitmapRectangle.UpRightY-WPG._2BitmapRectangle.LowLeftY);
-		 //break;
+                 Sx = (WPG._2BitmapRectangle.LowLeftX + WPG._2BitmapRectangle.UpRightX) / 2;
+                 Sy = (WPG._2BitmapRectangle.LowLeftY + WPG._2BitmapRectangle.UpRightY) / 2;
+                 RightX = WPG._2BitmapRectangle.UpRightX;                 
+                 TopY = WPG._2BitmapRectangle.UpRightY;
 		 }
 	      if(StartWPG.PosSizePrecision==1)
 		 {
 		 LoadWPG2DblBitmapRectangle(cq->wpd,WPG._2DblBitmapRectangle);
-		 ImgSts.x = WPG._2DblBitmapRectangle.LowLeftX/(float)0x10000;
-		 ImgSts.y = WPG._2DblBitmapRectangle.LowLeftY/(float)0x10000;
-		 ImgSts.dx = (WPG._2DblBitmapRectangle.UpRightX-WPG._2DblBitmapRectangle.LowLeftX) / (float)(0xFFFF);
-		 ImgSts.dy = (WPG._2DblBitmapRectangle.UpRightY-WPG._2DblBitmapRectangle.LowLeftY) / (float)(0xFFFF);
-		 //break;
+                 Sx = (WPG._2DblBitmapRectangle.LowLeftX + WPG._2DblBitmapRectangle.UpRightX) / (float)0x20000;
+                 Sy = (WPG._2DblBitmapRectangle.LowLeftY + WPG._2DblBitmapRectangle.UpRightY) / (float)0x20000;
+                 RightX = WPG._2DblBitmapRectangle.UpRightX / (float)0x10000;               
+                 TopY = WPG._2DblBitmapRectangle.UpRightY / (float)0x10000;
 		 }
-              // x=Transform(0,0);  dx=Transform(0,1)-x;
-	      // y=Transform(1,0);  dy=Transform(1,1)-y;
+              RightY = Sy;
+              TopX = Sx;
 
-	      if(TRx.ScaleX<1e-5) {ImgSts.x+=ImgSts.dx; ImgSts.dx*=-1;}	// Not all transforms are done. Not sure whether 2DblBitmapRectangle is before or after transform.
-	      if(TRx.ScaleY<1e-5) {ImgSts.y+=ImgSts.dy; ImgSts.dy*=-1;}
+              TRx.ApplyTransform(Sx,Sy);				// Image center is rotation invariant.
+              TRx.ApplyTransform(RightX,RightY);
+              TRx.ApplyTransform(TopX,TopY);
+
+              ImgSts.dx = 2 * sqrt((RightX-Sx)*(RightX-Sx) + (RightY-Sy)*(RightY-Sy));
+              ImgSts.dy = 2 * sqrt((TopX-Sx)*(TopX-Sx) + (TopY-Sy)*(TopY-Sy));
+              
+              if(fabs(RightY-Sy)<1e-5 && fabs(RightX-Sx)<1e-5)
+              {
+                ImgSts.RotAngle = TRx.RotAngle;
+                if(RightX < Sx) ImgSts.dx = -ImgSts.dx;	      
+                if(TopY < Sy) ImgSts.dy = -ImgSts.dy;
+              }
+              else
+              {
+                ImgSts.RotAngle = 180*atan2(RightY-Sy, RightX-Sx)/M_PI;  // Direct rot angle calculation according to all transforms.
+                if((fabs(ImgSts.RotAngle)<90) ^ (TopY>Sy))
+		    ImgSts.dy = -ImgSts.dy;
+              }
+
+	      ImgSts.x = Sx - fabs(ImgSts.dx);
+              ImgSts.y = Sy - fabs(ImgSts.dy);
 
 	      ImgSts.x /= 47.0;  ImgSts.dx /= 47.0;
 	      ImgSts.y /= 47.0;  ImgSts.dy /= 47.0;
@@ -1153,7 +1117,7 @@ WPG2_Transform TRx;
 	      }
     case 0x1C:{
 	      strcpy(cq->ObjType,"Text Line");
-	      LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+	      TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 	      ImgSts.RotAngle = TRx.RotAngle;
 
 	      LoadWPG2TextLine(cq->wpd,WPG._2TxtLine);
@@ -1163,7 +1127,7 @@ WPG2_Transform TRx;
 	      break;
 	      }
     case 0x1D:strcpy(cq->ObjType,"Text Block");
-              LoadWPG2Flags(cq->wpd,TRx,StartWPG.PosSizePrecision);
+              TRx.LoadWPG2Flags(cq->wpd,StartWPG.PosSizePrecision);
 	      ImgSts.RotAngle = TRx.RotAngle;
               if(StartWPG.PosSizePrecision==0)
               {
@@ -1310,7 +1274,7 @@ WPG2_Transform TRx;
     case 0x35:{WORD Pattern;
 	      Rd_word(cq->wpd,&Pattern);
 	      sprintf(cq->ObjType,"Brush Pattern %X", Pattern);
-	      if(Pattern==0) PSS.FillPattern = 1;
+	      if(Pattern==0) PSS.FillPattern = 1;	// Solid pattern.
 	      break;}
     case 0x36:{
               strcpy(cq->ObjType,"Horizontal Line");
@@ -1376,7 +1340,7 @@ WPG_records WPG;
 FILE *SrcImage;
 string NewFilename;
 Image Img,*CurrImg;
-DWORD ResourceEnd, NewObject, i;
+DWORD ResourceEnd, i;
 Raster2DAbstract *Raster=NULL;
 APalette *Palette=NULL;
 PS_State PSS = {0,0,0,0};
@@ -1439,7 +1403,7 @@ TryWPG1:
 	Rd_WP_DWORD(cq->wpd,&Rec.RecordLength);
 	if(feof(cq->wpd)) break;
 	cq->ActualPos=ftell(cq->wpd);
-	NewObject=cq->ActualPos+Rec.RecordLength;
+	DWORD NewObject = cq->ActualPos+Rec.RecordLength;
 
 	NumFormat=0;
 	switch(Rec.RecType)
@@ -1465,10 +1429,10 @@ TryWPG1:
 		      BYTE LineColor,c;
 		      WORD LineWidth;
 
-		      c=fgetc(cq->wpd);
+		      c = fgetc(cq->wpd);
 		      if(PSS.LineStyle!=c) {PSS.LineStyle=c;PSS.dirty|=PSS_LineStyle;}
-		      LineColor=fgetc(cq->wpd);
-		      PSS.LineColor=WPG_Palette[LineColor];
+		      LineColor = fgetc(cq->wpd);
+		      PSS.LineColor = WPG_Palette[LineColor];
 		      if(Palette!=NULL)
 			 {
 			 if(LineColor<Palette->Size1D)
@@ -1480,11 +1444,15 @@ TryWPG1:
 			 }
 		      PSS.dirty|=PSS_LineColor;
 		      Rd_word(cq->wpd,&LineWidth);
-		      if(PSS.LineWidth!=WPGu2PSu(2*LineWidth))
+		      if(fabs(PSS.LineWidth-WPGu2PSu(LineWidth)) > 1e-5)
 			{
-			PSS.LineWidth=WPGu2PSu(2*LineWidth);
-			PSS.dirty|=PSS_LineWidth;
+			PSS.LineWidth = WPGu2PSu(LineWidth);
+			PSS.dirty |= PSS_LineWidth;
 			}
+                      if(Verbosing>=2 && cq->log!=NULL)
+                        {
+                        fprintf(cq->log, "\nStyle:%u, Color:%u, Width: %u", (unsigned)c, (unsigned)LineColor, (unsigned)LineWidth);
+                        }
 		      break;
 		      }
 	  case  0x3:strcpy(cq->ObjType,"!Symbol Attributes"); break;
@@ -1571,7 +1539,7 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 		      ImgSts.RotAngle = ImgSts.x = ImgSts.y = ImgSts.dx = ImgSts.dy = 0;
 		      CurrImg->Raster=Raster;CurrImg->Raster->UsageCount++;Raster=NULL;
 		      if(CurrImg->Raster->GetPlanes()>1)
-                        AssignPalette(CurrImg,Palette);
+                          AssignPalette(CurrImg,Palette);
 		      break;
 	  case  0xC:strcpy(cq->ObjType,"!Graphics Text");NumFormat=1; break; //This is a bug fix for Draw Perfect WPG
 	  case  0xD:{strcpy(cq->ObjType,"Text Attributes");
@@ -1610,35 +1578,21 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 	  case 0x12:strcpy(cq->ObjType,"!Output Attributes"); break;
 	  case 0x13:{strcpy(cq->ObjType,"Plain Curve");
 		      fseek(cq->wpd, 4, SEEK_CUR);
-
-		      PS_Attr(PSData,&PSS);
+		      
 		      Rd_word(cq->wpd,&WPG.Curve.Count);
-                             FixPolySize(cq, &WPG.Curve.Count, Rec.RecordLength);
-		      WPG.Curve.x = Rd_word_MMX(cq->wpd,bbx.MinX,bbx.MaxX);
-		      WPG.Curve.y = Rd_word_MMX(cq->wpd,bbx.MinY,bbx.MaxY);
-		      PSData.cat_printf("\nnewpath\n%2.2f %2.2f moveto",WPGu2PSu(WPG.Curve.x),WPGu2PSu(WPG.Curve.y));
-		      if((WPG.Curve.Count-1)%3==0)
-			   while(WPG.Curve.Count>1)
-			    {
-			    WORD rx,ry,qx,qy;
-			    rx = Rd_word_MMX(cq->wpd,bbx.MinX,bbx.MaxX);
-			    ry = Rd_word_MMX(cq->wpd,bbx.MinY,bbx.MaxY);
-			    qx = Rd_word_MMX(cq->wpd,bbx.MinX,bbx.MaxX);
-			    qy = Rd_word_MMX(cq->wpd,bbx.MinY,bbx.MaxY);
-			    WPG.Curve.x = Rd_word_MMX(cq->wpd,bbx.MinX,bbx.MaxX);
-			    WPG.Curve.y = Rd_word_MMX(cq->wpd,bbx.MinY,bbx.MaxY);
-			    PSData.cat_printf("\n%2.2f %2.2f %2.2f %2.2f %2.2f %2.2f curveto",
-				WPGu2PSu(rx),WPGu2PSu(ry),WPGu2PSu(qx),WPGu2PSu(qy),
-				WPGu2PSu(WPG.Curve.x),WPGu2PSu(WPG.Curve.y) );
-			    WPG.Curve.Count-=3;
-			    }
-		      else while(WPG.Curve.Count-->1)
-			    {
-			    WPG.Curve.x = Rd_word_MMX(cq->wpd,bbx.MinX,bbx.MaxX);
-			    WPG.Curve.y = Rd_word_MMX(cq->wpd,bbx.MinY,bbx.MaxY);
-			    PSData.cat_printf("\n%2.2f %2.2f lineto",WPGu2PSu(WPG.Curve.x),WPGu2PSu(WPG.Curve.y));
-			    }
-		      PSData+="\nstroke";
+                      FixPolySize(cq, &WPG.Curve.Count, Rec.RecordLength);
+                      float *Points = LoadPoints(cq, WPG.Curve.Count, bbx);
+                      if(Points==NULL) break;
+
+                      VectorLine *pVecLine;
+		      if((WPG.Curve.Count-1)%3 == 0)
+                        pVecLine = new VectorCurve(Points, WPG.Curve.Count);                        
+		      else		// Curve is deffective, plot at least polyline.
+                        pVecLine = new VectorLine(Points, WPG.Curve.Count);
+
+                      pVecLine->AttribFromPSS(PSS);  
+                      pVecLine->Transform(vecResizeXY(WPGu2PSu(1)));
+                      VectList.AddObject(pVecLine);
 		      break;
 		      }
 	  case 0x14:{
@@ -1670,12 +1624,12 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 	  case 0x17:strcpy(cq->ObjType,"!Plan Perfect"); break;
 	  case 0x18:  // NumFormat=1; // default DR
 		      //  CrackObject(cq,NewObject);
-		      ldblk=Rec.RecordLength-21;
+		      ldblk = Rec.RecordLength-21;
 		      if(ldblk>0)
 			{
 			fseek(cq->wpd,cq->ActualPos+4,SEEK_SET);
 			Rd_word(cq->wpd,&WPG.TextL2.RotAngle);
-			WPG.TextL2.RotAngle=WPG.TextL2.RotAngle%360;
+			WPG.TextL2.RotAngle = WPG.TextL2.RotAngle%360;
 			fseek(cq->wpd,cq->ActualPos+8,SEEK_SET);
 			Rd_word(cq->wpd,(WORD *)&WPG.TextL2.LowLeftX);
 			Rd_word(cq->wpd,(WORD *)&WPG.TextL2.LowLeftY);
@@ -1693,12 +1647,12 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 		    if(Rec.RecordLength>0x3B)
 		      if(EmImg!=NULL)
 		        {
-		        EmbeddedImg *Ei2=(EmbeddedImg *)malloc(sizeof(EmbeddedImg));
-			Ei2->Offset=cq->ActualPos+0x3C;   /*skip PS l2 header in the wpg*/
-		        Ei2->Length=Rec.RecordLength-0x3C;
-		        Ei2->Extension=Ei2->ImgName=NULL;
-		        Ei2->Next=*EmImg;
-		        *EmImg=Ei2;
+		        EmbeddedImg *Ei2 = (EmbeddedImg *)malloc(sizeof(EmbeddedImg));
+			Ei2->Offset = cq->ActualPos+0x3C;   /*skip PS l2 header in the wpg*/
+		        Ei2->Length = Rec.RecordLength-0x3C;
+		        Ei2->Extension = Ei2->ImgName=NULL;
+		        Ei2->Next = *EmImg;
+		        *EmImg = Ei2;
 			}
 		      break;
 	  default: sprintf(cq->ObjType,"?%d?",(int)Rec.RecType); break;
@@ -1744,7 +1698,7 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
   if(Box.Image_type==2 && Box.Image_size>0) //WPG level 2 (might contain l.1 or different content)
     {
     WPG2Start StartWPG2;
-    float_matrix CTM(3,3);
+    //float_matrix CTM(3,3);
 
     if(Filename==NULL || *Filename==0)
         Filename=GetSomeImgName(".wpg");
@@ -1789,7 +1743,7 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
       while(cq->ActualPos<ResourceEnd)
 	{
 	RdDWORD_HiEnd(&Rec2.RecordLength, cq->wpd);
-	NewObject = ftell(cq->wpd) + Rec2.RecordLength;
+	DWORD NewObject = ftell(cq->wpd) + Rec2.RecordLength;
 	Rec2.Class = fgetc(cq->wpd);
 	Rec2.Type = fgetc(cq->wpd);
 
@@ -1861,11 +1815,11 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 	    Rd_WP_DWORD(cq->wpd,&Rec2.Extension);
 	    Rd_WP_DWORD(cq->wpd,&Rec2.RecordLength);
 	    cq->ActualPos = ftell(cq->wpd);
-	    NewObject = cq->ActualPos+Rec2.RecordLength;
+	    DWORD NewObject = cq->ActualPos+Rec2.RecordLength;
 
 	    if(feof(cq->wpd)) break;
 
-            ProcessWPG2Token(cq, Rec2, StartWPG2, Img, CTM, &Palette,
+            ProcessWPG2Token(cq, Rec2, StartWPG2, Img, /*CTM,*/ &Palette,
 	      PSData, PSS, EmImg, NewObject, ResourceEnd, &CurrImg,
 	      ImgSts, bbx, VectList);
 
@@ -1881,7 +1835,7 @@ LoadRaster:	      if(CurrImg->Raster!=NULL)
 
 	    if(NewObject!=ftell(cq->wpd))
 		    fseek(cq->wpd,NewObject,SEEK_SET);
-	    cq->ActualPos=NewObject;
+	    cq->ActualPos = NewObject;
 	    }
 
     if(PSData.length() > 0)
