@@ -10,12 +10,32 @@
 #include <math.h>
 
 #include "vecimage.h"
-#include "../images.h"
 
 
 static void FixPsAccent(string & s, const char *TexAccent, const char *PsAccent);
 
 
+/// Emit Postscript command for font selection.
+/// @param[in,out]	Text	String builter that holds whole PS image.
+/// @param[in]		FontSize  Size of font in [points]
+/// @param[in]		FontName  Optional naome of font.
+void Font2PS(string & Text, float FontSize, const char *FontName)
+{
+  Text.cat_printf("/%s findfont %2.2f scalefont setfont", FontName, FontSize);
+}
+
+
+/** Store color in PS notation. */
+void Color2PS(string & Text, const RGB_Record & TextColor)
+{
+  Text.cat_printf("%0.2g %0.2g %0.2g setrgbcolor",
+	         (float)TextColor.Red/256.0,
+		 (float)TextColor.Green/256.0,
+	         (float)TextColor.Blue/256.0);
+}
+
+
+
 ////////////////////////////////////
 
 vecPen::vecPen(void)
@@ -165,7 +185,7 @@ bool ContextSaved = false;
     s.cat_printf("\n%2.2f %2.2f translate", TranslateX, TranslateY);
   }
 
-  if(fabs(1-ScaleX)>1e-10 || fabs(1-ScaleY)>1e-10)
+  if(fabs(1-ScaleX)>1e-3 || fabs(1-ScaleY)>1e-3)
   {
     if(!ContextSaved)
     {
@@ -221,7 +241,14 @@ temp_string str;
 			 fabs(RightRect-LeftRect)/2, fabs(TopRect-BottomRect)/2,
 			 bAngle, eAngle);
 
-  FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);
+  if(BrushStyle!=0)
+  {
+    FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);
+  }
+  else
+  {    
+    str += "\nstroke";
+  }
 
   if(ContextSaved) str += "\ngrestore";
 
@@ -246,6 +273,9 @@ void VectorEllipse::Transform(const AbstractTransformXY &Tx)
     RightRect = LeftRect;
     LeftRect = f;
   }
+
+  if(this->Tx!=NULL)
+    Tx.ApplyTransform(this->Tx->CenterX, this->Tx->CenterY);
 }
 
 
@@ -270,6 +300,18 @@ VectorRectangle::~VectorRectangle()
 }
 
 
+unsigned VectorRectangle::isInside(float xp, float yp) const
+{
+  if(BrushStyle==0) return 0;
+  if(xp>=LeftRect && xp<=RightRect && yp>=BottomRect && xp<=TopRect)
+  {
+    if(xp>LeftRect && xp<RightRect && yp>BottomRect && xp<TopRect) return 2;
+    return 1;
+  }
+  return 0;  
+}
+
+
 temp_string VectorRectangle::Export2EPS(PS_State *PSS) const
 {
 temp_string str;
@@ -291,7 +333,14 @@ temp_string str;
   str.cat_printf("\n%2.2f %2.2f lineto", RightRect, BottomRect);
   str.cat_printf("\n%2.2f %2.2f lineto", LeftRect, BottomRect);
 
-  FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);
+  if(BrushStyle!=0)
+  {
+    FillObjectPS(LeftRect,RightRect,BottomRect,TopRect,str,PSS);    
+  }
+  else
+  {    
+    str += "\nstroke";
+  }  
 
   if(ContextSaved) str += "\ngrestore";
 
@@ -418,6 +467,22 @@ temp_string str;
     vecBrush::prepExport(PSS);
     vecPen::prepExport(PSS);    
     PS_Attr(str,PSS);
+    if(PSS->PolyFillMode==1 && UpperCont!=NULL && CountPoints>=3)
+    {
+      unsigned IntersectCount=0;
+      const VectorObject *vec = UpperCont->FirstObject;
+      while(vec!=NULL)
+      {
+        if(vec==this) break;
+        IntersectCount += vec->isInside(Points[0],Points[1]);
+        if(vec==UpperCont->LastObject) break;
+        vec = vec->NextObject;
+      }
+      if((IntersectCount & 3) == 2)
+      {
+        PSS->FirstTimeFix = true;	// Flip foreground to background.
+      }
+    }
   }
   
   const float *pEnd = Points + 2*CountPoints;
@@ -455,6 +520,284 @@ void VectorPolygon::Transform(const AbstractTransformXY &Tx)
 }
 
 
+/*unsigned VectorPolygon::isInside(float xp, float yp) const
+{
+  unsigned isX = isInsideX(xp,yp);
+  unsigned isY = isInsideY(xp,yp);
+
+  if(isX == isY) return isX; 
+  
+FILE *F;
+F=fopen("R:\\dump.csv","wb");
+const float *pEnd = Points + 2*(CountPoints);
+for(float *p=Points; p<pEnd; p+=2)
+{
+  fprintf(F,"%f,%f\n", p[0], p[1]);
+}
+fclose(F);
+
+  if(isX < isY) return isX;	// Trouble detected here.
+  return isY;
+}*/
+
+
+unsigned VectorPolygon::isInside(float xp, float yp) const
+{
+unsigned Intersects = 0;
+bool TouchLine = false;
+int OriUpDn = 0;
+
+  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==0) return 0;
+
+  const float *pEnd = Points + 2*(CountPoints-1);
+  for(float *p=Points; p<pEnd; p+=2)
+  {
+    if(fabs(p[1]-p[3]) < 1e-5)			// x axis segment; same y
+    {
+      if(fabs(p[1]-yp) < 1e-5)			// crossing the half-line
+      {
+        if(p[0]>xp || p[2]>xp) continue;
+        if(OriUpDn==0)
+        {
+          if(fabs(Points[1]-pEnd[1]) >= 1e-5)
+            OriUpDn = (Points[1]>pEnd[1]) ? 1 : -1;
+          else
+          {
+            for(int idx=2*(CountPoints-2)+1; idx>2; idx-=2)
+            {
+              if(fabs(Points[idx]-Points[idx+2]) >= 1e-5)
+              {
+                OriUpDn = (Points[idx+2]>Points[idx]) ? 1 : -1;
+                break;
+              }
+            }
+          }
+          TouchLine = true;
+        }        
+      }      
+      continue;
+    }    
+
+    const float t = (yp-p[1]) / (p[3]-p[1]);
+    const float xl = p[0] + (p[2]-p[0])*t;
+    if(xl > xp) {TouchLine=false;continue;}	// No intersection with half-line
+
+    if(fabs(yp-p[1])<1e-5 || fabs(yp-p[3])<1e-5) // Node intersection.
+    {
+      Intersects++;
+      if(TouchLine)
+      {
+        int NewOri = (p[3]>p[1]) ? 1 : -1;
+        if(NewOri!=OriUpDn)			// Up & Down touch
+          Intersects -= 2;			// This is NOT exit from an area.
+        TouchLine = false;
+      }
+      else
+        TouchLine = true;
+
+      OriUpDn = (p[3]>p[1]) ? 1 : -1;            
+      continue;
+    }
+
+    TouchLine = false;
+    if(t>1 || t<0) continue;			// No intersection with line segment.
+    Intersects += 2;
+  }
+
+  if(fabs(pEnd[1]-Points[1]) >= 1e-5)	 	// No x axis segment
+  {
+    const float t = (yp-pEnd[1]) / (pEnd[1]-Points[1]);
+    const float xl = pEnd[0] + t*(Points[0]-pEnd[0]);
+    if(xl < xp)
+    {
+      if(fabs(yp-pEnd[1])<1e-5 || fabs(yp-Points[1])<1e-5) // Node intersection.
+      {
+        Intersects++;        
+      }
+      else
+      {
+        if(t<=1 && t>=0) 
+		Intersects += 2;
+      }
+    }
+  }
+
+  return Intersects;
+}
+
+
+/*unsigned VectorPolygon::isInsideY(float xp, float yp) const
+{
+unsigned Intersects = 0;
+bool TouchLine = false;
+int OriUpDn = 0;
+
+  if(CountPoints<=2 || Points==NULL || !Close || BrushStyle==0) return 0;
+
+  const float *pEnd = Points + 2*(CountPoints-1);
+  for(float *p=Points; p<pEnd; p+=2)
+  {
+    if(fabs(p[0]-p[2]) < 1e-5)			// y axis segment; same x
+    {
+      if(fabs(p[0]-xp) < 1e-5)			// crossing the half-line
+      {
+        if(p[0]>xp || p[2]>xp) continue;
+        if(OriUpDn==0)
+        {
+          if(fabs(Points[0]-pEnd[0]) >= 1e-5)
+            OriUpDn = (Points[0]>pEnd[0]) ? 1 : -1;
+          else
+          {
+            for(int idx=2*(CountPoints-2); idx>2; idx-=2)
+            {
+              if(fabs(Points[idx]-Points[idx+2]) >= 1e-5)
+              {
+                OriUpDn = (Points[idx+2]>Points[idx]) ? 1 : -1;
+                break;
+              }
+            }
+          }
+          TouchLine = true;
+        }        
+      }      
+      continue;
+    }    
+
+    const float t = (xp-p[0]) / (p[2]-p[0]);
+    const float yl = p[1] + (p[3]-p[1])*t;
+    if(yl > yp) {TouchLine=false;continue;}	// No intersection with half-line
+
+    if(fabs(xp-p[0])<1e-5 || fabs(xp-p[2])<1e-5) // Node intersection.
+    {
+      Intersects++;
+      if(TouchLine)
+      {
+        int NewOri = (p[2]>p[0]) ? 1 : -1;
+        if(NewOri!=OriUpDn)			// Up & Down touch
+          Intersects -= 2;			// This is NOT exit from an area.
+        TouchLine = false;
+      }
+      else
+        TouchLine = true;
+
+      OriUpDn = (p[2]>p[0]) ? 1 : -1;            
+      continue;
+    }
+
+    TouchLine = false;
+    if(t>1 || t<0) continue;			// No intersection with line segment.
+    Intersects += 2;
+  }
+
+  if(fabs(pEnd[0]-Points[0]) >= 1e-5)	 	// No x axis segment
+  {
+    const float t = (xp-pEnd[0]) / (pEnd[0]-Points[0]);
+    const float yl = pEnd[1] + t*(Points[1]-pEnd[1]);
+    if(yl < yp)
+    {
+      if(fabs(xp-pEnd[0])<1e-5 || fabs(xp-Points[0])<1e-5) // Node intersection.
+      {
+        Intersects++;        
+      }
+      else
+      {
+        if(t<=1 && t>=0) Intersects += 2;
+      }
+    }
+  }
+
+  return Intersects;
+}
+*/
+
+
+
+/////////////////////////////////////////////////////////////
+
+
+temp_string VectorCurve::Export2EPS(PS_State *PSS) const
+{
+temp_string str;
+float MinPolyX,MaxPolyX,MinPolyY,MaxPolyY;
+bool ExtraStartup;
+float *p = Points;
+
+	// There must be at least two points for the line.
+  if(Points==NULL || CountPoints<=1) return str;
+
+  if((CountPoints-1)%3 == 0) 
+  {
+    ExtraStartup = true;    
+  }
+  else
+  {
+    if((CountPoints)%3!=0 || CountPoints<6) return VectorLine::Export2EPS(PSS);
+    ExtraStartup = false;
+    p += 2;		// point to anchor points.
+  }
+
+  if(PSS != NULL)
+  {    
+    vecPen::prepExport(PSS);    
+    PS_Attr(str,PSS);
+  }
+  
+  str.cat_printf("\nnewpath\n%2.2f %2.2f moveto", *p, p[1]);
+  MinPolyX = MaxPolyX = *p;	// Control point
+  MinPolyY = MaxPolyY = p[1];
+
+  const float *pEnd = Points + 2*CountPoints;
+  if(ExtraStartup)
+  {
+    for(p=Points+2; p<pEnd; p+=6)
+    {
+      str.cat_printf("\n%2.2f %2.2f %2.2f %2.2f %2.2f %2.2f curveto", *p, p[1], p[2], p[3], p[4], p[5]);
+
+      if(*p<MinPolyX) MinPolyX=*p;	if(*p>MaxPolyX) MaxPolyX=*p;
+      if(p[1]<MinPolyY) MinPolyY=p[1];	if(p[1]>MaxPolyY) MaxPolyY=p[1];
+
+      if(p[2]<MinPolyX) MinPolyX=p[2];	if(p[2]>MaxPolyX) MaxPolyX=p[2];
+      if(p[3]<MinPolyY) MinPolyY=p[3];	if(p[3]>MaxPolyY) MaxPolyY=p[3];
+
+      if(p[4]<MinPolyX) MinPolyX=p[4];	if(p[4]>MaxPolyX) MaxPolyX=p[4];
+      if(p[5]<MinPolyY) MinPolyY=p[5];	if(p[5]>MaxPolyY) MaxPolyY=p[5];
+    }
+  }
+  else
+  {
+    float xt_old = Points[4];		// Next control point.
+    float yt_old = Points[5];    
+    for(p=Points+6; p<pEnd; p+=6)
+    {
+      str.cat_printf("\n%2.2f %2.2f %2.2f %2.2f %2.2f %2.2f curveto", xt_old,yt_old, *p, p[1], p[2], p[3]);
+
+      if(*p<MinPolyX) MinPolyX=*p;	if(*p>MaxPolyX) MaxPolyX=*p;
+      if(p[1]<MinPolyY) MinPolyY=p[1];	if(p[1]>MaxPolyY) MaxPolyY=p[1];
+
+      if(p[2]<MinPolyX) MinPolyX=p[2];	if(p[2]>MaxPolyX) MaxPolyX=p[2];
+      if(p[3]<MinPolyY) MinPolyY=p[3];	if(p[3]>MaxPolyY) MaxPolyY=p[3];
+
+      xt_old = p[4];
+      yt_old = p[5];  
+      if(xt_old<MinPolyX) MinPolyX=xt_old;	if(xt_old>MaxPolyX) MaxPolyX=xt_old;
+      if(yt_old<MinPolyY) MinPolyY=yt_old;	if(yt_old>MaxPolyY) MaxPolyY=yt_old;
+    }
+  }
+
+  if(Filled && BrushStyle!=0)
+  {
+    str += "\nclosepath";
+    FillObjectPS(MinPolyX,MaxPolyX,MinPolyY,MaxPolyY,str,PSS);
+  }
+  else
+  {
+    if(Close) str += "\nclosepath";
+    str += "\nstroke"; 
+  }
+  return str;
+}
+
+
 /////////////////////////////////////////////////////////////
 
 VectorLine::VectorLine(int iniPointCount)
@@ -546,7 +889,7 @@ void PsBlob::FeaturesEPS(unsigned char & Feature) const
 VectorList::VectorList(void)
 {
   FirstObject = LastObject = NULL;
-  VectorObjects =0;
+  VectorObjects = 0;
 }
 
 
@@ -608,6 +951,7 @@ void VectorList::AddObject(VectorObject *NewObj)
 {
   if(NewObj==NULL || NewObj==this) return;
 
+  NewObj->UpperCont = this;
   if(FirstObject==NULL)
   {
     FirstObject = LastObject = NewObj;
@@ -641,7 +985,14 @@ VectorImage::VectorImage(VectorList &VecList, const PS_State & NewPSS)
   UsageCount = 0;
   FirstObject = VecList.FirstObject;	VecList.FirstObject = NULL;
   LastObject = VecList.LastObject;	VecList.LastObject = NULL;
-  VectorObjects = VecList.VectorObjects; VecList.VectorObjects =0;
+  VectorObjects = VecList.VectorObjects; VecList.VectorObjects = 0;
+
+  VectorObject *IterObject = FirstObject;
+  while(IterObject!=NULL)
+  {
+    IterObject->UpperCont = this;
+    IterObject = IterObject->NextObject;
+  }
 }
 
 
