@@ -5,6 +5,17 @@
 #include "raster.h"
 
 
+#define PSS_LineColor 1
+#define PSS_LineStyle 2
+#define PSS_LineWidth 4
+#define PSS_FillColor 8
+
+#define mm2PSu(x)   ( (float)(x)*71/25.4 )
+
+void Font2PS(string & Text, float FontSize, const char *FontName="Times-Roman");
+void Color2PS(string & Text, const RGB_Record & TextColor);
+
+
 class CpTranslator;
 
 typedef struct
@@ -18,6 +29,7 @@ typedef struct
 	char dirty;
 	char FillPattern;
 	BYTE LineStyle;
+        BYTE PolyFillMode;		///< 0-standard; 1-alternating
 
 	float LineWidth;
 	RGB_Record FillColor;
@@ -124,6 +136,8 @@ public:
 
 ///////////////////////////////////////////////////////////
 
+class VectorList;
+
 
 /** Base Vector object. */
 class VectorObject
@@ -132,10 +146,13 @@ public:
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const {return temp_string();}
   virtual void Transform(const AbstractTransformXY &Tx) {}
   virtual void FeaturesEPS(unsigned char & Feature) const {return;}
-  virtual ~VectorObject() {NextObject=NULL;}
-  VectorObject() {NextObject=NULL;}
+  virtual unsigned isInside(float xp, float yp) const {return 0;}
+
+  virtual ~VectorObject() {NextObject=NULL;UpperCont=NULL;}
+  VectorObject() {NextObject=NULL;UpperCont=NULL;}
 
   VectorObject *NextObject;
+  VectorList *UpperCont;
 };
 
 
@@ -204,6 +221,19 @@ public:
 };
 
 
+class VectorCurve: public VectorLine, public vecBrush
+{
+public:
+  VectorCurve(int iniPointCount): VectorLine(iniPointCount) {Filled=false;};
+  VectorCurve(float *iniBottomRect, int iniPointCount): VectorLine(iniBottomRect,iniPointCount) {Filled=false;};
+
+  void AttribFromPSS(const PS_State &PSS) {vecPen::AttribFromPSS(PSS);vecBrush::AttribFromPSS(PSS);}
+  virtual temp_string Export2EPS(PS_State *PSS=NULL) const;
+
+  bool Filled;
+};
+
+
 class VectorPolygon: public VectorObject, public vecPen, public vecBrush
 {
 public:
@@ -212,8 +242,12 @@ public:
 
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const;
   virtual void Transform(const AbstractTransformXY &Tx);
+  virtual unsigned isInside(float xp, float yp) const;
   void AttribFromPSS(const PS_State &PSS) {vecPen::AttribFromPSS(PSS);vecBrush::AttribFromPSS(PSS);}
 
+  //unsigned isInsideX(float xp, float yp) const;
+  //unsigned isInsideY(float xp, float yp) const;
+
   int CountPoints;
   float *Points;  
   bool Close;
@@ -229,6 +263,7 @@ public:
 
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const;
   virtual void Transform(const AbstractTransformXY &Tx);
+  virtual unsigned isInside(float xp, float yp) const;
   void AttribFromPSS(const PS_State &PSS) {vecPen::AttribFromPSS(PSS);vecBrush::AttribFromPSS(PSS);}
 
   vecTransform *Tx;
