@@ -16,10 +16,20 @@ typedef struct
 */
 
 
+#ifdef _WIN32
+  #define JOB_COUNT_TYPE	long
+#else
+  #define JOB_COUNT_TYPE	int
+#endif
+
+
+
+
 class JobBase
 {
 public:
-  static volatile long JobCount;
+
+  static volatile JOB_COUNT_TYPE JobCount;
 
   virtual void Run(void)  {};  
   JobBase();
