@@ -21,11 +21,12 @@
  #include <unistd.h>
 #endif
 
-#include <sets.h>
-#include <stringa.h>
-#include <lists.h>
+#include "sets.h"
+#include "stringa.h"
+#include "lists.h"
 
 #include "wp2latex.h"
+#include "struct.h"
 #include "cp_lib/cptran.h"
 
 
@@ -98,17 +99,19 @@ void initBox(TBox &Box)
 //++++++Procedures for manipulating with ATTR struct+++++++
 
 /* This procedure initialize an attr structure */
-void InitAttr(attribute & a)
+void attribute::InitAttr(void)
 {
- a.Math_Depth = a.Closed_Depth = a.Opened_Depth = 0;
+ Math_Depth = Closed_Depth = Opened_Depth = 0;
 }
 
 
 
-/** Open all attributes and feed result into string. */
+/** Open all attributes and feed result into string.
+ * @param[in,out]	a	Attribute list.
+ * @param[out]		s	Openning sequence stored in string. */
 void Open_All_Attr(attribute & a, string & s)
 {
- s = "";
+ s.erase();
  while(a.Opened_Depth<a.Closed_Depth)
    {
    const TStyleOpCl *stl = StyleOpCl + a.stack[a.Opened_Depth++];
@@ -132,7 +135,7 @@ void Open_All_Attr(attribute & a, FILE *f)
    const TStyleOpCl *stl = StyleOpCl + a.stack[a.Opened_Depth++];
       
    if(a.Math_Depth<=0)
-       fputs(stl->Open,f);
+       fputs(stl->Open, f);
    else
        fputs((stl->MathOpen==NULL)?stl->Open:stl->MathOpen, f);
    
@@ -144,10 +147,11 @@ void Open_All_Attr(attribute & a, FILE *f)
 
 /** Temporarily close all attributes and feed result into the string.
  * @param[in,out]	a	Attribute structure.
- * @param[out]		s	String to receive closing sequence. */
-void Close_All_Attr(attribute & a, string & s, int StartAttr)
+ * @param[out]		s	String to receive closing sequence.
+ * @param[in]		StartAttr Start position of attributes beeing opened. */
+void Close_All_Attr(attribute & a, string & s, unsigned StartAttr)
 {
- s = "";
+ s.erase();
  while(a.Opened_Depth>StartAttr)
      {
      const TStyleOpCl *stl = StyleOpCl + a.stack[--a.Opened_Depth];     
@@ -163,7 +167,7 @@ void Close_All_Attr(attribute & a, string & s, int StartAttr)
 
 /** This procedure temporary closes all attributes in the attr structure.
  * @param[in,out]	a	Attribute structure. */
-void Close_All_Attr(attribute & a, FILE *f, int StartAttr)
+void Close_All_Attr(attribute & a, FILE *f, unsigned StartAttr)
 {
  while(a.Opened_Depth>StartAttr)
      {
@@ -178,31 +182,44 @@ void Close_All_Attr(attribute & a, FILE *f, int StartAttr)
 }
 
 
-/** This procedure opens one attribut (lettertype) in the attr structure. */
+/** This procedure opens one attribut (lettertype) in the attr structure.
+ * @param[in,out]	a	Stack with all attributes opened.
+ * @param[in]		Attr	Attribute to be opened anew. */
 void AttrOn(attribute & a, char Attr)
 {
-int i;
+unsigned i;
 			/*Try to find an opened attribute in the stack */
  for(i=0; i<a.Closed_Depth; i++)
-	{
-        if(a.stack[i]==Attr)
-		{
-		return;
-                }
-	}
- a.stack[a.Closed_Depth++]=Attr;
+   {
+   if(a.stack[i]==Attr)
+     {
+     return;
+     }
+   }
+ if(a.Closed_Depth>MaxAttributes) return;	// attr stack overflow.
+ a.stack[a.Closed_Depth++] = Attr;
+ 
+ switch(Attr)
+ {
+   case Cyr_Font: Cyrillic = true; break;
+   case Heb_Font: cjHebrew = true; break;
+ }
 
 return;
 }
 
 
 /** This function checks whether the given attribute is on. */
-int IsAttrOn(attribute & a, char Attr)
+int IsAttrOn(const attribute & a, char Attr)
 {
-int pos;
+unsigned pos;
 	/* Try to find a currently closed attribute in the stack */
- for(pos=a.Closed_Depth-1; pos>=0; pos--)
-	if(a.stack[pos]==Attr) return(pos);
+ pos = a.Closed_Depth;
+ while(pos>0)	// better than: "for(pos=a.Closed_Depth-1; pos>=0; pos--)"
+ {
+   pos--;
+   if(a.stack[pos]==Attr) return(pos);
+ }
 
 return(-1);
 }
@@ -214,23 +231,16 @@ void _AttrOff(attribute & a, char Attr, string & s)
 {
 int pos;
 
- s="";
+ s.erase();
 
 	/* Try to find a currently closed attribute in the stack */
- pos = a.Closed_Depth-1;
- while(pos>=0)
-    {
-    if(a.stack[pos]==Attr) break;
-    pos--;
-    }
-
- if(pos<0) {
-        //printf(_("Attribute cannot be closed"));
-          return;
-          }
-
-	/* Close all opened attributes above closed & closed one */
- while(a.Opened_Depth>pos)
+ pos = a.Closed_Depth;
+ while(pos > 0)
+   {
+   pos--;
+   if(a.stack[pos]==Attr)
+     {		/* Close all opened attributes above closed & closed one */
+     while(a.Opened_Depth>pos)		// a.Opened_Depth is granted here tobe at least 1, because 1>0.
 	{
         const TStyleOpCl *stl = StyleOpCl + a.stack[--a.Opened_Depth];        
 	if(a.Math_Depth>a.Opened_Depth)
@@ -241,21 +251,25 @@ int pos;
         else 
           s += stl->Close;
 	}
+		/* Remove closed attribute from chain of clossed attributes */
+     if(a.Closed_Depth>0) a.Closed_Depth--;
+     while(pos<a.Closed_Depth)
+       {
+       a.stack[pos] = a.stack[pos+1];
+       pos++;
+       }
 
-	/* Remove closed attribute from chain of clossed attributes */
- a.Closed_Depth--;
- while(pos<a.Closed_Depth)
-	{
-	a.stack[pos]=a.stack[pos+1];
-	pos++;
-	}
+     return;
+     }
+   }
 
-return;
+ //printf(_("Attribute cannot be closed"));
+ return;
 }
 
 
 /**This procedure closes one attribut and write closing sequence into the file cq->strip*/
-void AttrOff(TconvertedPass1 *cq,char attr_num)
+void AttrOff(TconvertedPass1 *cq, char attr_num)
 {
 string s;
 
@@ -276,7 +290,7 @@ return;
 
 
 /**This procedure opens an attribute. Opening sequence is delayed and it is not written now.*/
-void Attr_ON(TconvertedPass1 *cq,unsigned char b)
+void Attr_ON(TconvertedPass1 *cq, unsigned char b)
 {
 #ifdef DEBUG
   fprintf(cq->log,"\n#Attr_ON(%c) ",b);fflush(cq->log);
@@ -341,15 +355,15 @@ RejectAttr:
 
 
 /** This procedure creates opening&closing sequence of attrs after that Base will have same items as A_new. */
-void AttrFit(attribute & Base, attribute & A_New, string & s)
+void AttrFit(attribute & Base, const attribute & A_New, string & s)
 {
 int i,pos;
 const char *ch;
 
- s="";
+ s.erase();
 
  Base.Closed_Depth=Base.Opened_Depth;
- for(pos=0;pos<Base.Opened_Depth; pos++)
+ for(pos=0; pos<Base.Opened_Depth; pos++)
    {
    if(A_New.stack[pos]!=Base.stack[pos] || pos>=A_New.Closed_Depth)
      {  //close all attrs Base[MaxPos] ... Base[Pos]
@@ -368,7 +382,7 @@ const char *ch;
      }
    }
 
- for(i=pos;i<A_New.Closed_Depth;i++)
+ for(i=pos; i<A_New.Closed_Depth; i++)
    {
    AttrOn(Base,A_New.stack[i]);
    }
@@ -418,7 +432,7 @@ return TexAttrib;
 
 
 /** This procedure changes a number of columns in document. */
-void Column(TconvertedPass1 *cq,signed char number)
+void Column(TconvertedPass1 *cq, signed char number)
 {
 #ifdef DEBUG
   fprintf(cq->log,"\n#Column(%d) ",(int)number);fflush(cq->log);
@@ -547,7 +561,7 @@ NoIndent:
   b=0;
   switch(WP)
 	{
-	case 0:abs=cq->tabpos[0];break;
+	case 0:abs=cq->tabpos[0]; break;
 	case 0x14:fread(&b, 1, 1, cq->wpd);	//obsolette WP4
 	       if(b>=TAB_POS_SIZE) b = TAB_POS_SIZE-1;
 	       abs = cq->tabpos[b];
@@ -712,7 +726,7 @@ void Make_tableentry_tabset(TconvertedPass1 *cq)
   fwrite(&b, sizeof(unsigned char), 1, cq->table);
 
   FORLIM = cq->num_of_tabs;
-  for (j = 0; j < FORLIM; j++)
+  for(j=0; j<FORLIM; j++)
     Wr_word(cq->table, cq->tabpos[j]);
 }
 
@@ -760,15 +774,18 @@ void NewLines(TconvertedPass1 *cq, int Lines, bool CRFlag)
 
 inline void SkipWP6Packets(TconvertedPass1 *cq)
 {
+#ifdef DEBUG
+  fprintf(cq->log,"\n#SkipWP6Packets() ");fflush(cq->log);
+#endif
 BYTE flags,PIDs;
 
-flags=fgetc(cq->wpd);
-if(feof(cq->wpd)) return;
-if(flags & 0x80)
-  {
-  PIDs=fgetc(cq->wpd);
-  fseek(cq->wpd,2*(int)PIDs,SEEK_CUR);
-  }
+ flags=fgetc(cq->wpd);
+ if(feof(cq->wpd)) return;
+ if(flags & 0x80)
+   {
+   PIDs = fgetc(cq->wpd);
+   fseek(cq->wpd,2*(int)PIDs,SEEK_CUR);
+   }
 }
 
 //++++++General Purpose procedures for converting files+++++++
@@ -1082,7 +1099,7 @@ void EndTable(TconvertedPass1 *cq)
 }
 
 
-/*This function converts Flush Right command*/
+/** This function converts Flush Right command. */
 void Flush_right(TconvertedPass1 *cq,char type)
 {
 #ifdef DEBUG
@@ -1563,7 +1580,7 @@ void ParagraphNumber(TconvertedPass1 *cq)
 		  else sprintf(cq->ObjType, "ParNum:%d",(int)ParNumType);
 
 /* check whether the section is opened */
-  for (i = 0; i < cq->attr.Closed_Depth; i++)
+  for(i = 0; i < cq->attr.Closed_Depth; i++)
 	{
 	if(cq->attr.stack[i] >= First_com_section &&
 	   cq->attr.stack[i] <= Last_com_section) return;
@@ -1585,7 +1602,7 @@ void ParagraphNumber(TconvertedPass1 *cq)
 
 		       if(ParNumType>=7) return;
 
-		       fprintf(cq->strip, "\\the%s ", Parts[ParNumType] );
+		       fprintf(cq->strip, "\\the%s ", Parts[ParNumType]);
 		       }
 
   cq->char_on_line = true;
@@ -2409,10 +2426,12 @@ if(type==6)
 }
 
 
+///////////////////////////////////////////
+
 
 void TconvertedPass1::InitMe(FILE *FileIn, FILE *Table, FILE *StripOut, FILE *LogFile,FILE *ErrorFile)
 {
-  InitAttr(attr);
+  attr.InitAttr();
 
   wpd   = FileIn;
   table = Table;
@@ -2430,5 +2449,11 @@ void TconvertedPass1::InitMe(FILE *FileIn, FILE *Table, FILE *StripOut, FILE *Lo
   Make_tableentry_tabset(this);   /* Geef the defaulttabinstelling door tied the 2e pass */
 }
 
+
+void ReportNoMemory(TconvertedPass1 *cq)
+{
+  fprintf((cq==NULL||cq->err==NULL)?stderr:cq->err, _("\nFatal: Not enough memory."));
+}
+
 /*--------------------End of PASS1--------------------*/
 
