@@ -287,6 +287,7 @@ protected:
      void parse_Chord(void);
      void parse_LineTo(VectorList &VectList);
      void parse_MoveTo(void);
+     void parse_SetPolyFillMode(void);
      void parse_Polygon(VectorList &VectList);
      void parse_PolyPolygon(VectorList &VectList);
      void parse_PolyLine(VectorList &VectList);
@@ -991,6 +992,25 @@ static const char ObjName[] = "!SetMapMode(%u)";
 }
 
 
+void TconvertedPass1_WMF::parse_SetPolyFillMode(void)
+{
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetPolyFillMode() ");fflush(log);
+#endif
+static const char ObjName[] = "!SetPolyFillMode(%u)";
+WORD PolyFillMode;
+  if(!CheckWmfRecSz(1))
+  {
+    ReportCorruptedObj(ObjName+1);
+    strncpy(ObjType,ObjName,16);
+    return;
+  }  
+  Rd_word(wpd, &PolyFillMode);
+  PSS.PolyFillMode = (PolyFillMode==1) ? 1 : 0;
+  sprintf(ObjType, ObjName+1, (unsigned)PolyFillMode);
+}
+
+
 void TconvertedPass1_WMF::parse_TextOut(VectorList &VectList)
 {
 #ifdef DEBUG
@@ -1315,7 +1335,7 @@ DWORD NewPos;
 	case 0x0103: parse_SetMapMode(); break;
 	case 0x0104: strcpy(ObjType,"!SetROP2"); break;
 	case 0x0105: strcpy(ObjType,"!SetRelabs"); break;
-	case 0x0106: strcpy(ObjType,"!SetPolyFillMode"); break;
+	case 0x0106: parse_SetPolyFillMode(); break;
 	case 0x0107: strcpy(ObjType,"!SetStretchBltMode"); break;
 	case 0x0108: strcpy(ObjType,"!SetTextCharExtra"); break;
 
@@ -1477,11 +1497,10 @@ DWORD NewPos;
       {        
         if(Img2->dy<0)
         {
-          Img2->y += Img2->dy;
-          Img2->dy = -Img2->dy;          
+          Img2->dy = -Img2->dy;
+          //Img2->y += Img2->dy;                    
         }
-        else
-          Flip2D(Img2->Raster);
+        Flip2D(Img2->Raster);
         if(Img2->dx<0)
         {        
          Img2->x += Img2->dx;
